@@ -19,8 +19,8 @@ use pixi_build_frontend::{
     json_rpc::CommunicationError,
 };
 use pixi_build_types::{
-    BackendCapabilities, BinaryPackageSpecV1, NamedSpecV1, PackageSpecV1, ProjectModelV1,
-    SourcePackageName, TargetSelectorV1, TargetV1, TargetsV1, VariantValue, VersionedProjectModel,
+    BackendCapabilities, BinaryPackageSpec, NamedSpec, PackageSpec, ProjectModel,
+    SourcePackageName, Target, TargetSelector, Targets, VariantValue,
     procedures::{
         conda_build_v1::{CondaBuildV1Params, CondaBuildV1Result},
         conda_outputs::{
@@ -32,7 +32,7 @@ use pixi_build_types::{
 };
 use rattler_conda_types::{
     PackageName, Platform, Version, VersionSpec,
-    package::{IndexJson, PathType, PathsEntry, PathsJson},
+    package::{IndexJson, PathType, PathsEntry, PathsJson, RunExportsJson},
 };
 use serde::Deserialize;
 
@@ -52,17 +52,20 @@ pub enum BackendEvent {
 /// backend is useful for testing and debugging purposes, as it does not perform
 /// any actual building or processing of the project model.
 pub struct PassthroughBackend {
-    project_model: ProjectModelV1,
+    project_model: ProjectModel,
     config: PassthroughBackendConfig,
     source_dir: PathBuf,
     index_json: IndexJson,
+    /// Run exports configuration for simulating package run_exports.
+    /// Maps package names to their run_exports definitions.
+    run_exports: BTreeMap<String, RunExportsJson>,
 }
 
 impl PassthroughBackend {
     /// Returns an object that can be used to instantiate a
     /// [`PassthroughBackend`].
-    pub fn instantiator() -> impl InMemoryBackendInstantiator<Backend = Self> {
-        PassthroughBackendInstantiator
+    pub fn instantiator() -> PassthroughBackendInstantiator {
+        PassthroughBackendInstantiator::default()
     }
 }
 
@@ -71,7 +74,6 @@ impl InMemoryBackend for PassthroughBackend {
         BackendCapabilities {
             provides_conda_outputs: Some(true),
             provides_conda_build_v1: Some(true),
-            ..BackendCapabilities::default()
         }
     }
 
@@ -85,7 +87,12 @@ impl InMemoryBackend for PassthroughBackend {
         _output_stream: &(dyn BackendOutputStream + Send + 'static),
     ) -> Result<CondaOutputsResult, Box<CommunicationError>> {
         // Generate outputs for all variant combinations
-        let outputs = generate_variant_outputs(&self.project_model, &self.index_json, &params);
+        let outputs = generate_variant_outputs(
+            &self.project_model,
+            &self.index_json,
+            &params,
+            &self.run_exports,
+        );
 
         Ok(CondaOutputsResult {
             outputs,
@@ -242,14 +249,17 @@ fn create_conda_package_on_the_fly(
     Ok(())
 }
 
-/// Generates all variant outputs for a package based on the variant configuration.
+/// Generates all variant outputs for a package based on the variant
+/// configuration.
 ///
-/// If any dependency has a "*" version requirement and there's a variant configuration
-/// for that package, multiple outputs will be generated - one for each variant combination.
+/// If any dependency has a "*" version requirement and there's a variant
+/// configuration for that package, multiple outputs will be generated - one for
+/// each variant combination.
 fn generate_variant_outputs(
-    project_model: &ProjectModelV1,
+    project_model: &ProjectModel,
     index_json: &IndexJson,
     params: &CondaOutputsParams,
+    run_exports: &BTreeMap<String, RunExportsJson>,
 ) -> Vec<CondaOutput> {
     // Check if we have variant configurations and dependencies with "*"
     let variant_keys = find_variant_keys(project_model, params);
@@ -261,6 +271,7 @@ fn generate_variant_outputs(
             index_json,
             params,
             BTreeMap::new(),
+            run_exports,
         )];
     }
 
@@ -283,6 +294,7 @@ fn generate_variant_outputs(
             index_json,
             params,
             BTreeMap::new(),
+            run_exports,
         )];
     }
 
@@ -292,12 +304,13 @@ fn generate_variant_outputs(
     // Create an output for each variant combination
     combinations
         .into_iter()
-        .map(|variant| create_output(project_model, index_json, params, variant))
+        .map(|variant| create_output(project_model, index_json, params, variant, run_exports))
         .collect()
 }
 
-/// Finds all dependency names that have "*" requirements and have variant configurations.
-fn find_variant_keys(project_model: &ProjectModelV1, params: &CondaOutputsParams) -> Vec<String> {
+/// Finds all dependency names that have "*" requirements and have variant
+/// configurations.
+fn find_variant_keys(project_model: &ProjectModel, params: &CondaOutputsParams) -> Vec<String> {
     let Some(targets) = &project_model.targets else {
         return Vec::new();
     };
@@ -309,7 +322,7 @@ fn find_variant_keys(project_model: &ProjectModelV1, params: &CondaOutputsParams
     let mut variant_keys = BTreeSet::new();
 
     // Helper to check dependencies in a target
-    let mut check_deps = |deps: Option<&OrderMap<SourcePackageName, PackageSpecV1>>| {
+    let mut check_deps = |deps: Option<&OrderMap<SourcePackageName, PackageSpec>>| {
         if let Some(deps) = deps {
             for (name, spec) in deps {
                 // Check if this dependency has a "*" requirement
@@ -346,13 +359,13 @@ fn find_variant_keys(project_model: &ProjectModelV1, params: &CondaOutputsParams
 }
 
 /// Checks if a package spec has a "*" version requirement.
-fn is_star_requirement(spec: &PackageSpecV1) -> bool {
-    let PackageSpecV1::Binary(boxed) = spec else {
+fn is_star_requirement(spec: &PackageSpec) -> bool {
+    let PackageSpec::Binary(boxed) = spec else {
         return false;
     };
 
-    match boxed.as_ref() {
-        BinaryPackageSpecV1 {
+    match boxed {
+        BinaryPackageSpec {
             version,
             build: None,
             build_number: None,
@@ -451,10 +464,11 @@ fn compute_variant_hash(variant: &BTreeMap<String, VariantValue>) -> String {
 
 /// Creates a single output with the given variant configuration.
 fn create_output(
-    project_model: &ProjectModelV1,
+    project_model: &ProjectModel,
     index_json: &IndexJson,
     params: &CondaOutputsParams,
     mut variant: BTreeMap<String, VariantValue>,
+    run_exports_config: &BTreeMap<String, RunExportsJson>,
 ) -> CondaOutput {
     let subdir = index_json
         .subdir
@@ -474,6 +488,37 @@ fn create_output(
         );
     }
 
+    // Extract explicit run dependencies
+    let mut run_dependencies = extract_dependencies(
+        &project_model.targets,
+        |t| t.run_dependencies.as_ref(),
+        params.host_platform,
+        &variant,
+    );
+
+    // Extract host dependencies
+    let host_deps = extract_dependencies(
+        &project_model.targets,
+        |t| t.host_dependencies.as_ref(),
+        params.host_platform,
+        &variant,
+    );
+
+    // Apply run_exports from host dependencies.
+    // For each host dependency that has run_exports configured, add the weak exports
+    // to run_dependencies with variant values substituted.
+    for host_dep in &host_deps.depends {
+        if let Some(pkg_run_exports) = run_exports_config.get(host_dep.name.as_str()) {
+            // Apply weak run_exports (most common case)
+            for weak_export in &pkg_run_exports.weak {
+                // Parse the run_export spec and substitute variant values
+                if let Some(pinned_spec) = resolve_run_export_spec(weak_export, &variant) {
+                    run_dependencies.depends.push(pinned_spec);
+                }
+            }
+        }
+    }
+
     CondaOutput {
         build_dependencies: Some(extract_dependencies(
             &project_model.targets,
@@ -481,18 +526,8 @@ fn create_output(
             params.host_platform,
             &variant,
         )),
-        host_dependencies: Some(extract_dependencies(
-            &project_model.targets,
-            |t| t.host_dependencies.as_ref(),
-            params.host_platform,
-            &variant,
-        )),
-        run_dependencies: extract_dependencies(
-            &project_model.targets,
-            |t| t.run_dependencies.as_ref(),
-            params.host_platform,
-            &variant,
-        ),
+        host_dependencies: Some(host_deps),
+        run_dependencies,
         metadata: CondaOutputMetadata {
             name: project_model
                 .name
@@ -522,8 +557,8 @@ fn create_output(
     }
 }
 
-fn extract_dependencies<F: Fn(&TargetV1) -> Option<&OrderMap<SourcePackageName, PackageSpecV1>>>(
-    targets: &Option<TargetsV1>,
+fn extract_dependencies<F: Fn(&Target) -> Option<&OrderMap<SourcePackageName, PackageSpec>>>(
+    targets: &Option<Targets>,
     extract: F,
     platform: Platform,
     variant: &BTreeMap<String, VariantValue>,
@@ -549,7 +584,7 @@ fn extract_dependencies<F: Fn(&TargetV1) -> Option<&OrderMap<SourcePackageName,
                     let resolved_spec = if is_star_requirement(spec) {
                         if let Some(variant_value) = variant.get(name.as_str()) {
                             // Replace with a version spec using the variant value
-                            PackageSpecV1::Binary(Box::new(BinaryPackageSpecV1 {
+                            PackageSpec::Binary(BinaryPackageSpec {
                                 version: Some(
                                     rattler_conda_types::VersionSpec::from_str(
                                         variant_value.to_string().as_str(),
@@ -558,7 +593,7 @@ fn extract_dependencies<F: Fn(&TargetV1) -> Option<&OrderMap<SourcePackageName,
                                     .unwrap(),
                                 ),
                                 ..Default::default()
-                            }))
+                            })
                         } else {
                             spec.clone()
                         }
@@ -566,7 +601,7 @@ fn extract_dependencies<F: Fn(&TargetV1) -> Option<&OrderMap<SourcePackageName,
                         spec.clone()
                     };
 
-                    NamedSpecV1 {
+                    NamedSpec {
                         name: name.clone(),
                         spec: resolved_spec,
                     }
@@ -580,21 +615,91 @@ fn extract_dependencies<F: Fn(&TargetV1) -> Option<&OrderMap<SourcePackageName,
     }
 }
 
-/// Returns true if the given [`TargetSelectorV1`] matches the specified
+/// Resolves a run_export spec string (like "sdl2 *") by substituting variant values.
+///
+/// If the spec contains a "*" version and there's a variant value for the package name,
+/// the version is replaced with the variant value. Otherwise returns the spec as-is.
+fn resolve_run_export_spec(
+    run_export_str: &str,
+    variant: &BTreeMap<String, VariantValue>,
+) -> Option<NamedSpec<PackageSpec>> {
+    // Parse the run_export string as a MatchSpec
+    let match_spec = rattler_conda_types::MatchSpec::from_str(
+        run_export_str,
+        rattler_conda_types::ParseStrictness::Lenient,
+    )
+    .ok()?;
+
+    let name = match_spec
+        .name
+        .as_ref()?
+        .as_exact()?
+        .as_source()
+        .to_string();
+
+    // Check if there's a variant value for this package
+    let version_spec = if match_spec
+        .version
+        .as_ref()
+        .is_none_or(|v| matches!(v, VersionSpec::Any))
+    {
+        // If version is "*" or unspecified, try to use the variant value
+        if let Some(variant_value) = variant.get(&name) {
+            Some(
+                VersionSpec::from_str(
+                    variant_value.to_string().as_str(),
+                    rattler_conda_types::ParseStrictness::Lenient,
+                )
+                .ok()?,
+            )
+        } else {
+            match_spec.version.clone()
+        }
+    } else {
+        match_spec.version.clone()
+    };
+
+    Some(NamedSpec {
+        name: SourcePackageName::from(name),
+        spec: PackageSpec::Binary(BinaryPackageSpec {
+            version: version_spec,
+            ..Default::default()
+        }),
+    })
+}
+
+/// Returns true if the given [`TargetSelector`] matches the specified
 /// `platform`.
-fn matches_target_selector(selector: &TargetSelectorV1, platform: Platform) -> bool {
+fn matches_target_selector(selector: &TargetSelector, platform: Platform) -> bool {
     match selector {
-        TargetSelectorV1::Unix => platform.is_unix(),
-        TargetSelectorV1::Linux => platform.is_linux(),
-        TargetSelectorV1::Win => platform.is_windows(),
-        TargetSelectorV1::MacOs => platform.is_osx(),
-        TargetSelectorV1::Platform(target_platform) => target_platform == platform.as_str(),
+        TargetSelector::Unix => platform.is_unix(),
+        TargetSelector::Linux => platform.is_linux(),
+        TargetSelector::Win => platform.is_windows(),
+        TargetSelector::MacOs => platform.is_osx(),
+        TargetSelector::Platform(target_platform) => target_platform == platform.as_str(),
     }
 }
 
 /// An implementation of the [`InMemoryBackendInstantiator`] that creates a
 /// [`PassthroughBackend`].
-pub struct PassthroughBackendInstantiator;
+#[derive(Default)]
+pub struct PassthroughBackendInstantiator {
+    /// Run exports configuration for simulating package run_exports.
+    /// Maps package names to their run_exports definitions.
+    run_exports: BTreeMap<String, RunExportsJson>,
+}
+
+impl PassthroughBackendInstantiator {
+    /// Adds run_exports configuration for a package.
+    pub fn with_run_exports(
+        mut self,
+        package_name: impl Into<String>,
+        run_exports: RunExportsJson,
+    ) -> Self {
+        self.run_exports.insert(package_name.into(), run_exports);
+        self
+    }
+}
 
 impl InMemoryBackendInstantiator for PassthroughBackendInstantiator {
     type Backend = PassthroughBackend;
@@ -604,10 +709,10 @@ impl InMemoryBackendInstantiator for PassthroughBackendInstantiator {
         params: InitializeParams,
     ) -> Result<Self::Backend, Box<CommunicationError>> {
         let project_model = match params.project_model {
-            Some(VersionedProjectModel::V1(project_model)) => project_model,
-            _ => {
+            Some(project_model) => project_model,
+            None => {
                 return Err(Box::new(CommunicationError::BackendError(
-                    BackendError::new("Passthrough backend only supports project model v1"),
+                    BackendError::new("Passthrough backend requires a project model"),
                 )));
             }
         };
@@ -618,7 +723,7 @@ impl InMemoryBackendInstantiator for PassthroughBackendInstantiator {
         };
 
         // Read the package file if it is specified, or create IndexJson for on_the_fly mode
-        let source_dir = params.source_dir.expect("Missing source directory");
+        let source_dir = params.source_directory.expect("Missing source directory");
         let index_json = match &config.package {
             Some(path) => {
                 let path = source_dir.join(path);
@@ -674,6 +779,7 @@ impl InMemoryBackendInstantiator for PassthroughBackendInstantiator {
             config,
             source_dir,
             index_json,
+            run_exports: self.run_exports.clone(),
         })
     }
 
@@ -725,6 +831,36 @@ impl BackendObserver {
         }
         events
     }
+
+    /// Waits for build events with a timeout.
+    /// Returns all build events received within the timeout period.
+    /// This is useful for tests where the build might happen asynchronously.
+    pub async fn wait_for_build_events(
+        &mut self,
+        timeout: std::time::Duration,
+    ) -> Vec<BackendEvent> {
+        let mut events = Vec::new();
+        let deadline = tokio::time::Instant::now() + timeout;
+
+        loop {
+            let remaining = deadline.saturating_duration_since(tokio::time::Instant::now());
+            if remaining.is_zero() {
+                break;
+            }
+
+            match tokio::time::timeout(remaining, self.receiver.recv()).await {
+                Ok(Some(event)) => {
+                    if matches!(event, BackendEvent::CondaBuildV1Called) {
+                        events.push(event);
+                    }
+                }
+                Ok(None) => break, // Channel closed
+                Err(_) => break,   // Timeout
+            }
+        }
+
+        events
+    }
 }
 
 /// An observable wrapper around any InMemoryBackend that emits events through a
@@ -736,8 +872,9 @@ pub struct ObservableBackend<T: InMemoryBackend> {
 }
 
 impl<T: InMemoryBackend> ObservableBackend<T> {
-    /// Creates a new instantiator for an ObservableBackend wrapping the given backend.
-    /// Returns both the instantiator and a BackendObserver for collecting events.
+    /// Creates a new instantiator for an ObservableBackend wrapping the given
+    /// backend. Returns both the instantiator and a BackendObserver for
+    /// collecting events.
     pub fn instantiator<I>(
         inner_instantiator: I,
     ) -> (
@@ -827,33 +964,34 @@ where
 
 #[cfg(test)]
 mod tests {
-    use super::*;
-    use pixi_build_types::{BinaryPackageSpecV1, PackageSpecV1};
+    use pixi_build_types::{BinaryPackageSpec, PackageSpec};
     use rattler_conda_types::{ParseStrictness, VersionSpec};
 
+    use super::*;
+
     #[test]
     fn test_is_star_requirement_with_star() {
-        let spec = PackageSpecV1::Binary(Box::new(BinaryPackageSpecV1 {
+        let spec = PackageSpec::Binary(BinaryPackageSpec {
             version: Some(VersionSpec::from_str("*", ParseStrictness::Lenient).unwrap()),
             ..Default::default()
-        }));
+        });
 
         assert!(is_star_requirement(&spec));
     }
 
     #[test]
     fn test_is_star_requirement_with_version() {
-        let spec = PackageSpecV1::Binary(Box::new(BinaryPackageSpecV1 {
+        let spec = PackageSpec::Binary(BinaryPackageSpec {
             version: Some(VersionSpec::from_str(">=1.0", ParseStrictness::Lenient).unwrap()),
             ..Default::default()
-        }));
+        });
 
         assert!(!is_star_requirement(&spec));
     }
 
     #[test]
     fn test_is_star_requirement_with_no_version() {
-        let spec = PackageSpecV1::Binary(Box::default());
+        let spec = PackageSpec::Binary(BinaryPackageSpec::default());
 
         assert!(is_star_requirement(&spec));
     }
