@@ -92,24 +92,24 @@ impl UpdateSpecs {
         package: LockedPackageRef<'_>,
     ) -> bool {
         // Check if the platform is in the list of platforms to update.
-        if let Some(platforms) = &self.platforms {
-            if !platforms.contains(platform) {
-                return false;
-            }
+        if let Some(platforms) = &self.platforms
+            && !platforms.contains(platform)
+        {
+            return false;
         }
 
         // Check if the environment is in the list of environments to update.
-        if let Some(environments) = &self.environments {
-            if !environments.contains(environment_name) {
-                return false;
-            }
+        if let Some(environments) = &self.environments
+            && !environments.contains(environment_name)
+        {
+            return false;
         }
 
         // Check if the package is in the list of packages to update.
-        if let Some(packages) = &self.packages {
-            if !packages.contains(package.name()) {
-                return false;
-            }
+        if let Some(packages) = &self.packages
+            && !packages.contains(package.name())
+        {
+            return false;
         }
 
         tracing::debug!(
@@ -162,8 +162,8 @@ pub async fn execute(args: Args) -> miette::Result<()> {
     let relaxed_lock_file = unlock_packages(&workspace, loaded_lock_file, &specs);
 
     // Update the packages in the lock-file.
-    let updated_lock_file = UpdateContext::builder(&workspace)
-        .with_lock_file(relaxed_lock_file)
+    let updated_lock_file = UpdateContext::builder(&workspace, None)?
+        .with_lock_file(relaxed_lock_file.clone())
         .with_no_install(args.no_install)
         .with_update_targets(specs.packages.clone())
         .finish()
@@ -215,10 +215,10 @@ fn ensure_package_exists(
     let environments = lock_file
         .environments()
         .filter_map(|(name, env)| {
-            if let Some(envs) = &specs.environments {
-                if !envs.contains(name) {
-                    return None;
-                }
+            if let Some(envs) = &specs.environments
+                && !envs.contains(name)
+            {
+                return None;
             }
             Some(env)
         })
@@ -228,10 +228,10 @@ fn ensure_package_exists(
         .iter()
         .flat_map(|env| env.packages_by_platform())
         .filter_map(|(p, packages)| {
-            if let Some(platforms) = &specs.platforms {
-                if !platforms.contains(&p) {
-                    return None;
-                }
+            if let Some(platforms) = &specs.platforms
+                && !platforms.contains(&p)
+            {
+                return None;
             }
             Some(packages)
         })
