@@ -30,6 +30,7 @@ use pixi_config::{Config, RunPostLinkScripts, default_channel_config, pixi_home}
 use pixi_consts::consts::{self};
 use pixi_core::repodata::Repodata;
 use pixi_manifest::PrioritizedChannel;
+use pixi_path::AbsPathBuf;
 use pixi_progress::global_multi_progress;
 use pixi_reporters::TopLevelProgress;
 use pixi_spec::{BinarySpec, PathBinarySpec};
@@ -508,6 +509,11 @@ impl Project {
         self.manifest.parsed.envs.get(name)
     }
 
+    /// Returns the path to the environment root directory.
+    pub fn env_root_path(&self) -> &Path {
+        self.env_root.path()
+    }
+
     /// Returns the EnvDir with the environment name.
     pub async fn environment_dir(&self, name: &EnvironmentName) -> miette::Result<EnvDir> {
         EnvDir::from_env_root(self.env_root.clone(), name).await
@@ -774,10 +780,9 @@ impl Project {
             if !package_executables
                 .iter()
                 .any(|executable| executable.name.as_str() == package_name.as_normalized())
+                && let Some(exec) = find_binary_by_name(&prefix, package_name).await?
             {
-                if let Some(exec) = find_binary_by_name(&prefix, package_name).await? {
-                    package_executables.push(exec);
-                }
+                package_executables.push(exec);
             }
 
             executables_for_package.insert(package_name.clone(), package_executables);
@@ -1352,11 +1357,17 @@ impl Project {
         self.command_dispatcher.get_or_try_init(|| {
             let multi_progress = global_multi_progress();
             let anchor_pb = multi_progress.add(ProgressBar::hidden());
-            let cache_dirs = pixi_command_dispatcher::CacheDirs::new(
-                pixi_config::get_cache_dir()
-                    .map(|cache_dir| cache_dir.join(BUILD_DIR))
-                    .map_err(|e| CommandDispatcherError::CacheDirectory(e.into()))?,
-            );
+            let cache_dir_path = pixi_config::get_cache_dir()
+                .map(|cache_dir| cache_dir.join(BUILD_DIR))
+                .map_err(|e| CommandDispatcherError::CacheDirectory(e.into()))?;
+            let cache_dir = AbsPathBuf::new(cache_dir_path)
+                .expect("cache dir is not absolute")
+                .into_assume_dir();
+            let cache_dirs = pixi_command_dispatcher::CacheDirs::new(cache_dir);
+
+            let root_dir = AbsPathBuf::new(self.root.clone())
+                .expect("root dir is not absolute")
+                .into_assume_dir();
 
             Ok(pixi_command_dispatcher::CommandDispatcher::builder()
                 .with_gateway(
@@ -1365,7 +1376,7 @@ impl Project {
                         .clone(),
                 )
                 .with_cache_dirs(cache_dirs)
-                .with_root_dir(self.root.clone())
+                .with_root_dir(root_dir)
                 .with_download_client(
                     self.authenticated_client()
                         .map_err(|e| CommandDispatcherError::AuthenticatedClient(e.into()))?
