@@ -374,6 +374,41 @@ impl Feature {
     pub fn pypi_options(&self) -> Option<&PypiOptions> {
         self.pypi_options.as_ref()
     }
+
+    /// Returns the dev dependencies of the feature for a given `platform`.
+    ///
+    /// Dev dependencies are source packages whose build/host/run dependencies
+    /// should be installed without building the packages themselves.
+    ///
+    /// This function returns a [`Cow`]. If the dependencies are not combined or
+    /// overwritten by multiple targets than this function returns a
+    /// reference to the internal dependencies.
+    ///
+    /// Returns `None` if this feature does not define any target that has any
+    /// of the requested dev dependencies.
+    ///
+    /// If the `platform` is `None` no platform specific dependencies are taken
+    /// into consideration.
+    pub fn dev_dependencies(
+        &self,
+        platform: Option<Platform>,
+    ) -> Option<Cow<'_, DependencyMap<PackageName, pixi_spec::SourceSpec>>> {
+        self.targets
+            .resolve(platform)
+            // Get the targets in reverse order, from least specific to most specific.
+            // This is required because we want more specific targets to overwrite their specs.
+            .rev()
+            .filter_map(|t| t.dev_dependencies.as_ref())
+            .filter(|deps| !deps.is_empty())
+            .fold(None, |acc, deps| match acc {
+                None => Some(Cow::Borrowed(deps)),
+                Some(acc) => {
+                    // Overwrite the accumulator with specs from this target
+                    // More specific targets (processed later) overwrite less specific ones
+                    Some(Cow::Owned(acc.as_ref().overwrite(deps)))
+                }
+            })
+    }
 }
 
 #[cfg(test)]
