@@ -11,8 +11,8 @@ def test_rich():
     assert version("rich").split(".")[0] == "13"
 
 
-def test_requests():
-    assert version("requests") == "2.31.0"
+def test_httpx():
+    assert version("httpx") == "0.28.1"
 
 
 def test_minimal_project():
